@@ -20,12 +20,11 @@ sys.path.insert(0, os.path.abspath(os.path.join('..', '..')))
 # -- Project information -----------------------------------------------------
 
 project = 'MacSyFinder'
-copyright = "2014-2022, 'Institut Pasteur (Paris), CNRS'"
+copyright = "2014-2024, 'Institut Pasteur (Paris), CNRS'"
 author = u"Sophie Abby, Bertrand Néron"
 
 # The full version, including alpha/beta/rc tags
-import macsypy
-release = macsypy.__version__
+release = '2.0'
 
 
 # -- General configuration ---------------------------------------------------
