@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2024  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -33,14 +33,18 @@ import sys
 import shutil
 import os.path
 import math
+
 from .report import GembaseHMMReport, GeneralHMMReport, OrderedHMMReport
 from .utils import threads_available
 from .database import Indexes
+from .config import Config
+from .gene import ModelGene, CoreGene
+from .report import HMMReport
 
 _log = logging.getLogger(__name__)
 
 
-def worker_cpu(genes_nb, cfg):
+def worker_cpu(genes_nb: int, cfg: Config) -> tuple[int, int]:
     """
     Compute the optimum number of worker and cpu per worker
     The number of worker is set by the user (1 by default 0 means all worker available)
@@ -49,9 +53,8 @@ def worker_cpu(genes_nb, cfg):
     if number of workers is greater than number of genes then several cpu can be use by
     hmsearch to speed up the search step
 
-    :param int genes_nb: the number of genes to search
+    :param genes_nb: the number of genes to search
     :param cfg: The macsyfinder configuration
-    :type cfg: :class:`macsypy.config.Config` object
     :return: the number of worker and cpu_per_worker to use
     :rtype: tuple (int worker_nb, int cpu_per_worker)
     """
@@ -62,7 +65,7 @@ def worker_cpu(genes_nb, cfg):
     return worker_nb, cpu_per_worker
 
 
-def search_genes(genes, cfg):
+def search_genes(genes: list[ModelGene], cfg: Config) -> list[HMMReport]:
     """
     For each gene of the list, use the corresponding profile to perform an Hmmer search, and parse the output
     to generate a HMMReport that is saved in a file after CoreHit filtering.
@@ -71,15 +74,13 @@ def search_genes(genes, cfg):
     in the command-line with the "-w" option.
 
     :param genes: the genes to search in the input sequence dataset
-    :type genes: list of :class:`macsypy.gene.ModelGene` objects
     :param cfg: the configuration object
-    :type cfg: :class:`macsypy.config.Config` object
     """
     worker_nb, cpu_per_worker = worker_cpu(len(genes), cfg)
     _log.debug(f"worker_nb = {worker_nb:d}\tcpu per worker = {cpu_per_worker}")
     all_reports = []
 
-    def stop(signum, frame):
+    def stop(signum: int, frame) -> None:
         """stop the main process, its threads and subprocesses"""
         _log.critical('KILL all Processes')
         proc_grp_id = os.getpgid(0)
@@ -91,13 +92,13 @@ def search_genes(genes, cfg):
     default_signal_handler = signal.signal(signal.SIGTERM, stop)
 
 
-    def search(gene, cpu):
+    def search(gene: CoreGene, cpu: int) -> HMMReport:
         """
         Search gene in the database built from the input sequence file (execute \"hmmsearch\"), and produce a HMMReport
 
         :param gene: the gene to search
-        :type gene: a :class:`macsypy.gene.CoreGene` object
-        :param int cpu: the number of cpu to use (per worker)
+        :param cpu: the number of cpu to use (per worker)
+        :return: an HMMReport
         """
         _log.info(f"search gene {gene.name}")
         profile = gene.profile
@@ -113,16 +114,13 @@ def search_genes(genes, cfg):
                 return report
 
 
-    def recover(gene, cfg):
+    def recover(gene: CoreGene, cfg: Config) -> HMMReport:
         """
         Recover Hmmer output from a previous run, and produce a report
 
         :param gene: the gene to search
-        :type gene: a :class:`macsypy.gene.CoreGene` object
         :param cfg: the configuration
-        :type cfg: :class:`macsypy.config.Config` object
         :return: the list of all HMMReports (derived class depending on the input dataset type)
-        :rtype: list of `macsypy.report.HMMReport` object
         """
         hmm_old_path = os.path.join(cfg.previous_run(), cfg.hmmer_dir(), gene.name + cfg.res_search_suffix())
         _log.info(f"recover hmm {hmm_old_path}")
