@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2024  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -26,7 +26,6 @@ import os
 import sys
 import shutil
 import unittest
-import platform
 from io import StringIO
 from contextlib import contextmanager
 import hashlib
@@ -125,7 +124,7 @@ class MacsyTest(unittest.TestCase):
 
     @staticmethod
     def remove_red_ansi_color(colored_msg):
-        red_pattern = "^\\x1b\[0?1;31m(.*)\\x1b\[0m$"
+        red_pattern = r"^\x1b\[0?1;31m(.*)\x1b\[0m$"
         msg = re.match(red_pattern, colored_msg).groups()[0]
         return msg
 
@@ -175,7 +174,6 @@ class MacsyTest(unittest.TestCase):
 
                 # the system_id may change from one run to another
                 # So I have to remove them before to compare each row
-                filename = os.path.basename(fh1.name)
                 if header is None:
                     header = fields_1[:]
 
@@ -266,7 +264,7 @@ class MacsyTest(unittest.TestCase):
         """
         try:
             shutil.rmtree(path)
-        except:
+        except Exception:
             pass
 
     @staticmethod
@@ -316,28 +314,3 @@ class LoggerWrapper(object):
 
     def get_value(self):
         return self.logger.handlers[0].stream.getvalue()
-
-
-def which(name, flags=os.X_OK):
-    """
-    Search PATH for executable files with the given name.
-
-    :param name: the name of the executable to search
-    :type name: str
-    :param flags: os mod the name must have, default is executable (os.X_OK).
-    :type flags: os file mode R_OK|R_OK|W_OK|X_OK
-    :return: the path of the executable
-    :rtype: string or None
-    """
-    result = None
-    path = os.environ.get('PATH', None)
-    if path is None:
-        return result
-    for p in os.environ.get('PATH', '').split(os.pathsep):
-        p = os.path.join(p, name)
-        if platform.system() == 'Windows':
-            p += '.exe'
-        if os.access(p, flags):
-            result = p
-            break
-    return result
