@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2024  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -53,7 +53,7 @@ class TestConfig(MacsyTest):
         try:
             shutil.rmtree(self.tmp_dir)
             #pass
-        except Exception as err:
+        except Exception:
             pass
 
     def test_str_2_tuple(self):
@@ -218,7 +218,7 @@ class TestConfig(MacsyTest):
                     self.assertEqual(getattr(cfg, opt)(), self.defaults['system_models_dir'])
                 else:
                     self.assertEqual(getattr(cfg, opt)(), val)
-        except:
+        except Exception:
             os.chdir(self._current_dir)
 
 
