@@ -2,7 +2,7 @@
 # MacSyFinder - Detection of macromolecular systems in protein dataset  #
 #               using systems modelling and similarity search.          #
 # Authors: Sophie Abby, Bertrand Neron                                  #
-# Copyright (c) 2014-2023  Institut Pasteur (Paris) and CNRS.           #
+# Copyright (c) 2014-2024  Institut Pasteur (Paris) and CNRS.           #
 # See the COPYRIGHT file for details                                    #
 #                                                                       #
 # This file is part of MacSyFinder package.                             #
@@ -64,7 +64,7 @@ class TestModel(MacsyTest):
     def clean_working_dir(self):
         try:
             shutil.rmtree(self.cfg.working_dir())
-        except:
+        except Exception:
             pass
 
     def test_fqn(self):
@@ -331,4 +331,4 @@ sctC
         model_buz = Model('Foo/buz', 10)
         self.assertTrue(isinstance(hash(model_bar), int))
         self.assertEqual(hash(model_bar), hash(model_bar_bis))
-        self.assertNotEqual(hash(model_bar), hash(model_buz))
\ No newline at end of file
+        self.assertNotEqual(hash(model_bar), hash(model_buz))
